use futures::Future;
use nostr_rs_client::{Clients, Error as ClientError};
use nostr_rs_storage::Db;
use nostr_rs_types::{client::Subscribe, Request, Response};
use std::pin::Pin;
use tokio::sync::mpsc;

#[derive(Debug, thiserror::Error)]
pub enum Error {
    #[error("Nostr: {0}")]
    Addr(#[from] nostr_rs_types::types::addr::Error),

    #[error("client: {0}")]
    Client(#[from] ClientError),
}

fn on_connection(socket: mpsc::Sender<Request>) -> Pin<Box<dyn Future<Output = ()> + Send>> {
    Box::pin(async move {
        log::info!("Connected to relay, subscribing to all events");
        let _ = socket.send(Subscribe::default().into()).await;
    })
}

#[tokio::main]
async fn main() {
    env_logger::init();
    let clients = Clients::default();

    let _ = clients
        .connect_to("wss://relay.damus.io/", Some(on_connection))
        .await;
    let db = Db::new("./db").expect("db");
    loop {
        if let Some((msg, _relayed_by)) = clients.recv().await {
            if let Response::Event(x) = msg {
                let event = x.event;

                let _ = db.store(&event);
            }
        }
    }
}
